
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define ERROR -1

#define NAME_LEN        1024

#define D64_LEN         174848
#define SECTOR_LEN      0x100

unsigned char image[D64_LEN];

char image_name[NAME_LEN];
char output_name[NAME_LEN];

int trackoffs[35+1] = {
    0,
0x00000,
0x01500,
0x02A00,
0x03F00,
0x05400,
0x06900,
0x07E00,
0x09300,
0x0A800,
0x0BD00,
0x0D200,
0x0E700,
0x0FC00,
0x11100,
0x12600,
0x13B00,
0x15000,

0x16500,
0x17800,
0x18B00,
0x19E00,
0x1B100,
0x1C400,
0x1D700,
0x1EA00,
0x1FC00,
0x20E00,
0x22000,
0x23200,
0x24400,
0x25600,
0x26700,
0x27800,
0x28900,
0x29A00
};

int tracksecs[35+1] = {
    0,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,
21,

19,
19,
19,
19,
19,
19,
19,

18,
18,
18,
18,
18,
18,

17,
17,
17,
17,
17
};

long flength(FILE *fh)
{
long len,pos;

    pos=ftell(fh);
    fseek(fh,0,SEEK_END);
    len=ftell(fh);
    fseek(fh,pos,SEEK_SET);
    return(len);
}

int random_in_range (unsigned int min, unsigned int max)
{
  int base_random = rand(); /* in [0, RAND_MAX] */
  if (RAND_MAX == base_random) return random_in_range(min, max);
  /* now guaranteed to be in [0, RAND_MAX) */
  int range       = max - min,
      remainder   = RAND_MAX % range,
      bucket      = RAND_MAX / range;
  /* There are range buckets, plus one smaller interval
     within remainder of RAND_MAX */
  if (base_random < RAND_MAX - remainder) {
    return min + base_random/bucket;
  } else {
    return random_in_range (min, max);
  }
}

unsigned char *getptr(int trk, int sec)
{
    return &image[trackoffs[trk]+(sec * SECTOR_LEN)];
}

unsigned char buffer[SECTOR_LEN];

void scrambleimage(void)
{
    unsigned char *p1a;
    unsigned char *p1b;
    int trk1,sec1,secs1;
    int trk1b,sec1b;
    unsigned char *p2a;
    unsigned char *p2b;
    int trk2,sec2,secs2;
    int trk2b,sec2b;
    int i;
    
    for (i=0;i<10000;i++) {
    
    do {
        do {
            while ( (trk1 = random_in_range (1, 35+1)) == 18) {
                // get random track, except 18
            }
            secs1 = tracksecs[trk1];
            // get random sector
            sec1 = random_in_range (0, secs1);
            // repeat until we found a sector that is NOT the end of a file
            p1a = getptr(trk1,sec1);
        } while (p1a[0] == 0);
        trk1b = p1a[0];
        sec1b = p1a[1];
        p1b = getptr(trk1b,sec1b);
    } while ((trk1b == 18) || (trk1b > 35) || (sec1b >= tracksecs[trk1b]));
    
    do {
        do {
            while ( (trk2 = random_in_range (1, 35+1)) == 18) {
                // get random track, except 18
            }
            secs2 = tracksecs[trk2];
            // get random sector
            sec2 = random_in_range (0, secs2);
            // repeat until we found a sector that is NOT the end of a file
            p2a = getptr(trk2,sec2);
        } while (p2a[0] == 0);
        trk2b = p2a[0];
        sec2b = p2a[1];
        p2b = getptr(trk2b,sec2b);
    } while ((trk2b == 18) || (trk2b > 35) || (sec2b >= tracksecs[trk2b]));
    
    // swap links to blocks
    p1a[0]=trk2b;
    p1a[1]=sec2b;
    p2a[0]=trk1b;
    p2a[1]=sec1b;
    // swap blocks
    memcpy(buffer, p1b, SECTOR_LEN);
    memcpy(p1b, p2b, SECTOR_LEN);
    memcpy(p2b, buffer, SECTOR_LEN);
    
//    printf("%2d,%2d (%2d,%2d) <-> ", trk1, sec1, trk1b, sec1b);
//    printf("%2d,%2d (%2d,%2d) -> ", trk2, sec2, trk2b, sec2b);
//    printf("%2d,%2d (%2d,%2d) <-> ", trk1, sec1, trk2b, sec2b);
//    printf("%2d,%2d (%2d,%2d)\n", trk2, sec2, trk1b, sec1b);
    }
}

//------------------------------------------------------------------------------

void loadd64file(char *name)
{
unsigned int len;
FILE *infile;

    infile = fopen(name, "rb");
    if (!infile) {
        fprintf(stderr, "Failed to open input file '%s'\n", name);
        exit(ERROR);
    }

    len = flength(infile);
    if (len != D64_LEN) {
        fprintf(stderr, "File '%s' length != %d bytes.\n", name, D64_LEN);
        exit(ERROR);
    }

    fread(image, 1, D64_LEN, infile);

    fclose(infile);
}

//-------------------------------------------------------------------------------
void saved64file(char *name)
{
FILE *outfile;

    outfile = fopen(name, "wb");
    if (!outfile) {
        fprintf(stderr, "Failed to open output file '%s'\n", name);
        exit(ERROR);
    }

    fwrite(image, 1, D64_LEN, outfile);

    fclose(outfile);
}

//-------------------------------------------------------------------------------

void usage(void)
{
    printf(
        "d64shredder v0.1 (w)2012 Groepaz/Hitmen\n"
        "usage: d64shredder [options]\n\n"
        "[options]\n"
        "-i [infile]     input d64 image\n"
        "-o [outfile]    output d64 image\n"
        "-?,-h           show this help screen\n"
    );
    exit(ERROR);
}

//-------------------------------------------------------------------------------

int main(int argc, char **argv){
int i;

    if(argc<5) {
        usage();
    }

    for(i=1;i<argc;i++){

        if((argv[i][0]=='-')){

            switch(argv[i][1]){
                case 'i': /* infile name */
                    i++;
                    strcpy(image_name, argv[i]);
                break;
                case 'o': /* outfile name */
                    i++;
                    strcpy(output_name, argv[i]);
                break;
            default:
                printf("error: unknown switch \"%s\"\n\n",argv[i]);
                case '?':
                case 'h':
                    usage();
                    break;
            }
        } else {
            usage();
        }
    }

// - load source d64
    loadd64file(image_name);
// screw up :)
    scrambleimage();
// - save output d64
    saved64file(output_name);
    
    return(0);
}
